use crate::utils::net_utils;
use std::net::{Ipv4Addr, Ipv6Addr};

#[derive(thiserror::Error, Debug)]
enum NatDefError {
    #[error("Invalid nat definition: {0}")]
    InvalidNatDef(&'static str),
}

#[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
#[serde(tag = "type", rename_all = "lowercase")]
pub enum NatSource<IPv> {
    Interface { name: String },
    Ip { ip: IPv },
}

#[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
pub enum NatProtocol {
    TCP,
    UDP,
    Both,
}

#[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
#[serde(tag = "type", rename_all = "lowercase")]
pub enum NatHostPort {
    Single { port: u16 },
    Range { start: u16, end: u16 },
}

#[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
pub struct Nat<IPv> {
    pub protocol: NatProtocol,
    pub host_addr: NatSource<IPv>,
    pub host_port: NatHostPort,
    pub guest_addr: IPv,
    pub guest_port: u16,
    pub comment: Option<String>,
}

impl<IPv> Nat<IPv> {
    pub fn check(&self) -> anyhow::Result<()> {
        if let NatSource::Interface { name } = &self.host_addr {
            if !net_utils::is_net_interface_name_valid(name) {
                return Err(NatDefError::InvalidNatDef("Invalid nat interface name!").into());
            }
        }

        if let NatHostPort::Range { start, end } = &self.host_port {
            if *start == 0 {
                return Err(NatDefError::InvalidNatDef("Invalid start range!").into());
            }

            if start > end {
                return Err(NatDefError::InvalidNatDef("Invalid port range!").into());
            }

            if u16::MAX - (end - start) < self.guest_port {
                return Err(NatDefError::InvalidNatDef("Guest port is too high!").into());
            }
        }

        if self.guest_port == 0 {
            return Err(NatDefError::InvalidNatDef("Invalid guest port!").into());
        }

        Ok(())
    }
}

#[derive(Debug, Clone, serde::Serialize, serde::Deserialize, Default)]
pub struct NetNat {
    pub interface: String,
    pub ipv4: Option<Vec<Nat<Ipv4Addr>>>,
    pub ipv6: Option<Vec<Nat<Ipv6Addr>>>,
}
